﻿' (C) Copyright 2011 by Autodesk, Inc. 
'
' Permission to use, copy, modify, and distribute this software
' in object code form for any purpose and without fee is hereby
' granted, provided that the above copyright notice appears in
' all copies and that both that copyright notice and the limited
' warranty and restricted rights notice below appear in all
' supporting documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK,
' INC. DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL
' BE UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is
' subject to restrictions set forth in FAR 52.227-19 (Commercial
' Computer Software - Restricted Rights) and DFAR 252.227-7013(c)
' (1)(ii)(Rights in Technical Data and Computer Software), as
' applicable.
'
Imports System
Imports System.Collections.Generic
Imports System.Linq

Imports Autodesk.Revit.DB
Imports Autodesk.Revit.DB.Architecture
Imports Autodesk.Revit.UI
Imports Autodesk.Revit.UI.Selection
Imports Autodesk.Revit.ApplicationServices
Imports Autodesk.Revit.Attributes


' Lab1 can duplicate a specified group to the position user picked.
' It shows the usage of following basic APIs 
' 
'  Get application and document objects.
'  Pick an object
'  Pick a point 
'  Transaction usage
'  Place group 
'  
' Before run this command, a group should  already be in the 
' active document.
'

<TransactionAttribute(TransactionMode.Manual), _
 RegenerationAttribute(RegenerationOption.Manual)> _
Public Class Lab1PlaceGroup
  Implements IExternalCommand

  Public Function Execute(ByVal commandData As ExternalCommandData, _
                          ByRef message As String, _
                          ByVal elements As ElementSet) As Result _
                          Implements IExternalCommand.Execute
    'Get application and document objects
    Dim uiApp As UIApplication = commandData.Application
    Dim doc As Document = uiApp.ActiveUIDocument.Document

    'Define a Reference object to accept the pick result.
    Dim pickedRef As Reference = Nothing
    'Pick a group
    Dim sel As Selection = uiApp.ActiveUIDocument.Selection
    pickedRef = sel.PickObject(ObjectType.Element, _
                               "Please select a group")
    'Get picked Element
    'In Revit 2011 we can get element directly by this line
    Dim elem As Element = pickedRef.Element
    'In Revit 2012, Reference.Element property is obsolete. 
    'The picked element can be retrieved by this:
    'Dim elem as Element = doc.GetElement(pickedRef)

    Dim group As Group = CType(elem, Group)

    'pick a point
    Dim point As XYZ = sel.PickPoint( _
    "Please pick a point to place group")

    'place the group
    Dim trans As Transaction = New Transaction(doc)
    trans.Start("Lab")
    doc.Create.PlaceGroup(point, group.GroupType)
    trans.Commit()

    Return Result.Succeeded
  End Function
End Class

